'use strict';

/**
 * ncc上下文信息
 * @author xg
 */
var _ = require('lodash');
var sa = require('superagent');
var Cookies = require('js-cookie');
var IsIE = require('./IsIE');
var noCache = require('./NoCache');

var NccContext = {
    /**
     * 初始化ncc上下文信息
     * @param options 参数配置{
     *  url:获取上下文的URL
     *  success:初始化上下文成功时的回调
     *  error:初始化上下文失败时的回调
     * }
     */
    init: function init(options) {
        var _ref = options || {},
            url = _ref.url,
            _success = _ref.success,
            _error = _ref.error;

        if (!url) {
            console.error("初始化ncc上下文失败: url为空");
            return;
        }
        var params = {
            "busiParamJson": "{}",
            "sysParamJson": {}
        };

        this.request({
            url: url,
            method: 'POST',
            send: params,
            cookies: true,
            noToken: true,
            success: function success(result) {
                if (result && result.success) {
                    // 写入Cookie
                    var cookies = result.data || {};
                    for (var key in cookies) {
                        if (cookies.hasOwnProperty(key)) {
                            Cookies.set(key, cookies['' + key]);
                        }
                    }
                    if (_success && _.isFunction(_success)) {
                        _success(result);
                    }
                }
            },
            error: function error() {
                if (_error && _.isFunction(_error)) {
                    _error();
                } else {
                    console.error("初始化ncc上下文失败");
                }
            }
        });
    },

    _defaultHeader: {},

    //设置全局缺省配置
    _defaultOptions: {
        method: 'GET', // 默认请求方法
        type: 'json', // 请求体发送方式 application/json|json|png
        url: '', // 请求url
        header: null, //{object}设置当前请求的header信息
        cookies: false, //是否携带cookies信息
        noToken: false, // 是否在请求中禁用token信息
        //reloginIfTokenExpired: true, // 是否token失效自动跳转到登录界面
        send: '', //POST请求参数，即参数不会拼接在URL后面。
        query: '', //GET请求参数，即参数拼接在URL后面。
        // type : '', // 默认content-type，支持text/html|application/x-www-form-urlencoded|application/json|multipart/form-data
        // accept : '', // 默认接受类型application/json|json|text|text/plain
        success: null, // 请求成功的回调函数
        error: null, // 请求失败的回调函数，包括http客户端错误、服务端错误、以及接口调用成功但存在业务错误
        complete: null // 请求完成的回调函数(不管成功还是失败)
    },

    request: function request(options) {
        if (options.success === undefined) delete options.success;
        if (options.error === undefined) delete options.error;
        if (options.complete === undefined) delete options.complete;
        if (options.cookies === undefined) delete options.cookies;
        var _defaults = _.assign({}, this._defaultOptions, options);
        // 禁用token
        if (options.query && options.query.noToken !== undefined) {
            _defaults.noToken = options.query.noToken;
        }
        // 发送ajax请求
        var contentType = 'application/json;charset=UTF-8';
        if (_defaults.type === 'form') {
            contentType = 'application/x-www-form-urlencoded;charset=UTF-8';
        }
        var finalNoCache = IsIE === -1 ? noCache : noCache.withQueryStrings;
        var req = sa(_defaults.method, _defaults.url).use(finalNoCache).set('Content-Type', contentType);
        //设置全局的header信息
        for (var header in this._defaultHeader) {
            if (this._defaultHeader[header] != undefined) {
                req.set(header, this._defaultHeader[header]);
            }
        }
        //设置当前请求的header信息
        if (_defaults.header && _.isPlainObject(_defaults.header)) {
            for (var headerKey in _defaults.header) {
                if (_defaults.header[headerKey] != undefined) {
                    req.set(headerKey, _defaults.header[headerKey]);
                }
            }
        }
        //设置请求携带cookie
        if (_defaults.cookies == true) {
            req.withCredentials();
        }
        req.query(_defaults.query).send(_defaults.send).end(function (err, res) {
            if (err || res && res.badRequest) {
                if (typeof _defaults.error == 'function') {
                    //如果有外部的错误异常处理则使用外部的
                    _defaults.error(res);
                } else {
                    //如果没有则使用内部默认的异常处理
                    // alert('请求异常[状态码:'+err.status+']\n'+err.message);
                    console.error('请求异常[状态码:' + err.status + ']', err);
                }
            }
            if (res && res.ok) {
                if (res.headers && res.headers['icop-content-type']) {
                    if (typeof _defaults.error == 'function') {
                        //如果有外部的错误异常处理则使用外部的
                        _defaults.error(res);
                    } else {
                        //如果没有则使用内部默认的异常处理
                        //捕获服务端包装的异常消息
                        alert('异常错误\n' + (res.text && res.text.length > 0 ? res.text : res.body));
                        console.error('异常错误', res);
                    }
                    return;
                } else {
                    if (typeof _defaults.success == 'function') {
                        if (_defaults.accept == 'text') {
                            _defaults.success(res.text);
                        } else {
                            _defaults.success(res.body);
                        }
                    }
                }
            }
            if (_defaults.complete) {
                _defaults.complete(res);
            }
        });
    }
};
module.exports = NccContext;